# -*- coding: utf-8 -*-
import math
from abjad.tools import datastructuretools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools import sequencetools
from abjad.tools.topleveltools import inspect_
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker


class EvenDivisionRhythmMaker(RhythmMaker):
    r'''Even division rhythm-maker.

    Object model of a partially evaluated function that accepts a (possibly
    empty) list of divisions as input and returns a list of selections as
    output (structured one selection per input division).
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Rhythm-makers'

    __slots__ = (
        '_burnish_specifier',
        '_denominators',
        '_extra_counts_per_division',
        '_preferred_denominator',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        denominators=[8],
        beam_specifier=None,
        burnish_specifier=None,
        logical_tie_masks=None,
        division_masks=None,
        duration_spelling_specifier=None,
        extra_counts_per_division=None,
        preferred_denominator='from_counts',
        tie_specifier=None,
        tuplet_spelling_specifier=None,
        ):
        from abjad.tools import rhythmmakertools
        RhythmMaker.__init__(
            self,
            beam_specifier=beam_specifier,
            logical_tie_masks=logical_tie_masks,
            duration_spelling_specifier=duration_spelling_specifier,
            division_masks=division_masks,
            tie_specifier=tie_specifier,
            tuplet_spelling_specifier=tuplet_spelling_specifier,
            )
        assert mathtools.all_are_nonnegative_integer_powers_of_two(
            denominators), repr(denominators)
        denominators = tuple(denominators)
        self._denominators = denominators
        if extra_counts_per_division is not None:
            assert mathtools.all_are_integer_equivalent(
                extra_counts_per_division), repr(extra_counts_per_division)
            extra_counts_per_division = [
                int(_) for _ in extra_counts_per_division
                ]
            extra_counts_per_division = tuple(extra_counts_per_division)
        self._extra_counts_per_division = extra_counts_per_division
        prototype = (rhythmmakertools.BurnishSpecifier, type(None))
        assert isinstance(burnish_specifier, prototype)
        self._burnish_specifier = burnish_specifier
        extra_counts_per_division = extra_counts_per_division or (0,)
        self._preferred_denominator = preferred_denominator

    ### SPECIAL METHODS ###

    def __call__(self, divisions, rotation=None):
        r'''Calls even division rhythm-maker on `divisions`.

        ..  container:: example

            Fills divisions with alternating eighth and sixteenth notes:

            ::


                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[8, 16],
                ...     )

            ::

                >>> divisions = [(3, 8), (3, 8), (3, 8), (3, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        {
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        {
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        {
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                }

        ..  container:: example

            Adds extra counts per division according to a pattern of three
            elements:

            ::


                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[16],
                ...     extra_counts_per_division=[0, 1, 2],
                ...     )

            ::

                >>> divisions = [(3, 8), (3, 8), (3, 8), (3, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 6/7 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 6/8 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 6/7 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                }

        Returns list of of selections.
        '''
        superclass = super(EvenDivisionRhythmMaker, self)
        return superclass.__call__(
            divisions,
            rotation=rotation,
            )

    ### PRIVATE METHODS ###

    def _apply_burnish_specifier(self, selections, rotation):
        if self.burnish_specifier is None:
            return selections
        left_classes = self.burnish_specifier.left_classes
        middle_classes = self.burnish_specifier.middle_classes
        right_classes = self.burnish_specifier.right_classes
        left_counts = self.burnish_specifier.left_counts
        right_counts = self.burnish_specifier.right_counts
        left_classes = left_classes or ()
        left_classes = sequencetools.Sequence(left_classes).rotate(n=rotation)
        left_classes = datastructuretools.CyclicTuple(left_classes)
        if middle_classes == () or middle_classes is None:
            middle_classes = (0,)
        middle_classes = sequencetools.Sequence(
            middle_classes).rotate(n=rotation)
        middle_classes = datastructuretools.CyclicTuple(middle_classes)
        right_classes = right_classes or ()
        right_classes = sequencetools.Sequence(
            right_classes).rotate(n=rotation)
        right_classes = datastructuretools.CyclicTuple(right_classes)
        left_counts = left_counts or (0,)
        left_counts = sequencetools.Sequence(left_counts).rotate(n=rotation)
        left_counts = datastructuretools.CyclicTuple(left_counts)
        right_counts = right_counts or (0,)
        right_counts = sequencetools.Sequence(right_counts).rotate(n=rotation)
        right_counts = datastructuretools.CyclicTuple(right_counts)
        if self.burnish_specifier.outer_divisions_only:
            procedure = self._burnish_outer_selections
        else:
            procedure = self._burnish_each_selection
        selections = procedure(
            selections,
            left_classes,
            middle_classes,
            right_classes,
            left_counts,
            right_counts,
            )
        return selections

    def _burnish_division_part(self, division_part, token):
        assert len(division_part) == len(token)
        new_division_part = []
        for leaf, burnishing in zip(division_part, token):
            if burnishing in (-1, scoretools.Rest):
                new_division_part.append(scoretools.Rest(leaf))
            elif burnishing == 0:
                new_division_part.append(leaf)
            elif burnishing in (1, scoretools.Note):
                new_division_part.append(scoretools.Note(leaf))
            else:
                raise ValueError
        new_division_part = type(division_part)(new_division_part)
        return new_division_part

    def _burnish_each_selection(
        self,
        selections,
        left_classes,
        middle_classes,
        right_classes,
        left_counts,
        right_counts,
        ):
        lefts_index, rights_index = 0, 0
        for selection_index, selection in enumerate(selections):
            tuplet = selection[0]
            original_duration = inspect_(tuplet).get_duration()
            leaves = tuplet[:]
            leaf_count = len(leaves)
            left_length = left_counts[selection_index]
            left = left_classes[lefts_index:lefts_index + left_length]
            lefts_index += left_length
            right_length = right_counts[selection_index]
            right = right_classes[rights_index:rights_index + right_length]
            rights_index += right_length
            available_left_length = leaf_count
            left_length = min([left_length, available_left_length])
            available_right_length = leaf_count - left_length
            right_length = min([right_length, available_right_length])
            middle_length = leaf_count - left_length - right_length
            left = left[:left_length]
            middle = middle_length * [middle_classes[selection_index]]
            right = right[:right_length]
            left_part, middle_part, right_part = sequencetools.Sequence(
                    leaves).partition_by_counts(
                    [left_length, middle_length, right_length],
                    cyclic=False,
                    overhang=False,
                    )
            left_part = self._burnish_division_part(left_part, left)
            middle_part = self._burnish_division_part(middle_part, middle)
            right_part = self._burnish_division_part(right_part, right)
            burnished_leaves = left_part + middle_part + right_part
            tuplet[:] = burnished_leaves
            assert inspect_(tuplet).get_duration() == original_duration
        return selections

    def _burnish_outer_selections(
        self,
        selections,
        left_classes,
        middle_classes,
        right_classes,
        left_counts,
        right_counts,
        ):
        if len(selections) == 1:
            self._burnish_each_selection(
                selections,
                left_classes,
                middle_classes,
                right_classes,
                left_counts,
                right_counts,
                )
            return selections
        left_length = 0
        if left_counts:
            left_length = left_counts[0]
        left = left_classes[:left_length]
        right_length = 0
        if right_counts:
            right_length = right_counts[0]
        right = right_classes[:right_length]

        # first selection
        tuplet = selections[0][0]
        original_duration = inspect_(tuplet).get_duration()
        leaves = tuplet[:]
        available_left_length = len(leaves)
        left_length = min([left_length, available_left_length])
        middle_length = len(leaves) - left_length
        left = left[:left_length]
        if not middle_classes:
            middle_classes = [1]
        middle = [middle_classes[0]]
        middle = middle_length * middle
        left_part, middle_part = sequencetools.Sequence(
                leaves).partition_by_counts(
                [left_length, middle_length],
                cyclic=False,
                overhang=False,
                )
        left_part = self._burnish_division_part(left_part, left)
        middle_part = self._burnish_division_part(middle_part, middle)
        burnished_leaves = left_part + middle_part
        tuplet[:] = burnished_leaves
        assert inspect_(tuplet).get_duration() == original_duration

        # middle selections
        for selection in selections[1:-1]:
            tuplet = selection[0]
            original_duration = inspect_(tuplet).get_duration()
            leaves = tuplet[:]
            middle = len(leaves) * [middle_classes[0]]
            burnished_leaves = self._burnish_division_part(leaves, middle)
            tuplet[:] = burnished_leaves
            assert inspect_(tuplet).get_duration() == original_duration

        # last selection
        tuplet = selections[-1][0]
        original_duration = inspect_(tuplet).get_duration()
        leaves = tuplet[:]
        available_right_length = len(leaves)
        right_length = min([right_length, available_right_length])
        middle_length = len(leaves) - right_length
        right = right[:right_length]
        middle = middle_length * [middle_classes[0]]
        middle_part, right_part = sequencetools.Sequence(
                leaves).partition_by_counts(
                [middle_length, right_length],
                cyclic=False,
                overhang=False,
                )
        middle_part = self._burnish_division_part(middle_part, middle)
        right_part = self._burnish_division_part(right_part, right)
        burnished_leaves = middle_part + right_part
        tuplet[:] = burnished_leaves
        assert inspect_(tuplet).get_duration() == original_duration
        return selections

    def _make_music(self, divisions, rotation):
        if rotation is None:
            rotation = 0
        selections = []
        divisions = [mathtools.NonreducedFraction(_) for _ in divisions]
        denominators = datastructuretools.CyclicTuple(self.denominators)
        extra_counts_per_division = self.extra_counts_per_division or (0,)
        extra_counts_per_division = datastructuretools.CyclicTuple(
            extra_counts_per_division
            )
        for i, division in enumerate(divisions, rotation):
            # not yet extended to work with non-power-of-two divisions
            if not mathtools.is_positive_integer_power_of_two(
                division.denominator):
                message = 'non-power-of-two divisions not implemented: {!r}.'
                message = message.format(division)
                raise Exception(message)
            denominator = denominators[i]
            extra_count = extra_counts_per_division[i]
            basic_duration = durationtools.Duration(1, denominator)
            unprolated_note_count = None
            if division < 2 * basic_duration:
                notes = scoretools.make_notes([0], [division])
            else:
                unprolated_note_count = division / basic_duration
                unprolated_note_count = int(unprolated_note_count)
                unprolated_note_count = unprolated_note_count or 1
                if 0 < extra_count:
                    modulus = unprolated_note_count
                    extra_count = extra_count % modulus
                elif extra_count < 0:
                    modulus = int(math.ceil(unprolated_note_count / 2.0))
                    extra_count = abs(extra_count) % modulus
                    extra_count *= -1
                note_count = unprolated_note_count + extra_count
                durations = note_count * [basic_duration]
                notes = scoretools.make_notes([0], durations)
                assert all(
                    _.written_duration.denominator == denominator
                    for _ in notes
                    )
            tuplet_duration = durationtools.Duration(division)
            tuplet = scoretools.FixedDurationTuplet(
                duration=tuplet_duration,
                music=notes,
                )
            if (self.preferred_denominator == 'from_counts' and
                unprolated_note_count is not None):
                preferred_denominator = unprolated_note_count
                tuplet.preferred_denominator = preferred_denominator
            elif isinstance(self.preferred_denominator, int):
                tuplet.preferred_denominator = self.preferred_denominator
            selection = selectiontools.Selection(tuplet)
            selections.append(selection)
        selections = self._apply_burnish_specifier(selections, rotation)
        beam_specifier = self._get_beam_specifier()
        beam_specifier(selections)
        selections = self._apply_division_masks(selections, rotation)
        return selections

    ### PUBLIC PROPERTIES ###

    @property
    def burnish_specifier(self):
        r'''Gets burnish specifier.

        ..  container:: example

            Forces the first leaf and the last two leaves to be rests:

            ::

                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     burnish_specifier=rhythmmakertools.BurnishSpecifier(
                ...         left_classes=[Rest],
                ...         left_counts=[1],
                ...         right_classes=[Rest],
                ...         right_counts=[2],
                ...         outer_divisions_only=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        {
                            r8
                            c'8 [
                            c'8 ]
                        }
                    }
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8 ]
                            r8
                            r8
                        }
                    }
                }

            Burnishing outer divisions also works when given a single division:

            ::

                >>> divisions = [(7, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 7/8
                        {
                            r8
                            c'8 [
                            c'8
                            c'8
                            c'8 ]
                            r8
                            r8
                        }
                    }
                }

        ..  container:: example

            Forces the first leaf of every division to be a rest:

            ::

                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     burnish_specifier=rhythmmakertools.BurnishSpecifier(
                ...         left_classes=[Rest],
                ...         left_counts=[1],
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        {
                            r8
                            c'8 [
                            c'8 ]
                        }
                    }
                    {
                        \time 4/8
                        {
                            r8
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 3/8
                        {
                            r8
                            c'8 [
                            c'8 ]
                        }
                    }
                    {
                        \time 4/8
                        {
                            r8
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                }

        Returns burnish specifier or none.
        '''
        return self._burnish_specifier

    @property
    def denominators(self):
        r'''Gets denominators.

        ..  container:: example

            Fills divisions with 16th notes:

            ::


                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[16],
                ...     )

            ::

                >>> divisions = [(3, 16), (3, 8), (3, 4)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/16
                        {
                            c'16 [
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 3/4
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                }

        ..  container:: example

            Fills divisions with 8th notes:

            ::


                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[8],
                ...     )

            ::

                >>> divisions = [(3, 16), (3, 8), (3, 4)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/16
                        {
                            c'8.
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 3/4
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                }

            Fills divisions less than twice the duration of an eighth note with
            a single attack.

        ..  container:: example

            Fills divisions with quarter notes:

            ::


                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[4],
                ...     )

            ::

                >>> divisions = [(3, 16), (3, 8), (3, 4)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/16
                        {
                            c'8.
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'4.
                        }
                    }
                    {
                        \time 3/4
                        {
                            c'4
                            c'4
                            c'4
                        }
                    }
                }

            Divisions less than twice the duration of a quarter note are filled
            with a single attack.

        ..  container:: example

            Fills divisions with half notes:

            ::


                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[2],
                ...     )

            ::

                >>> divisions = [(3, 16), (3, 8), (3, 4)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/16
                        {
                            c'8.
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'4.
                        }
                    }
                    {
                        \time 3/4
                        {
                            c'2.
                        }
                    }
                }

            Fills divisions less than twice the duration of a half note with a
            single attack.

        Returns tuple of nonnegative integer powers of two.
        '''
        if self._denominators:
            return list(self._denominators)

    @property
    def division_masks(self):
        r'''Gets division masks.

        ..  container:: example

            No division masks:

            ::

                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker()

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                }

        ..  container:: example

            Silences every other division:

            ::

                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     division_masks=[
                ...         rhythmmakertools.silence_every([0], period=2),
                ...         ],
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        r2
                    }
                    {
                        \time 3/8
                        {
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 4/8
                        r2
                    }
                    {
                        \time 3/8
                        {
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                }

        ..  container:: example

            Sustains every other division:

            ::

                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     division_masks=[
                ...         rhythmmakertools.sustain_every([0], period=2),
                ...         ],
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        c'2
                    }
                    {
                        \time 3/8
                        {
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 4/8
                        c'2
                    }
                    {
                        \time 3/8
                        {
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                }

        ..  container:: example

            Silences every output division:

            ::

                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     division_masks=rhythmmakertools.silence_all(),
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        r2
                    }
                    {
                        \time 3/8
                        r4.
                    }
                    {
                        \time 4/8
                        r2
                    }
                    {
                        \time 3/8
                        r4.
                    }
                }

        Set to division masks or none.
        '''
        superclass = super(EvenDivisionRhythmMaker, self)
        return superclass.division_masks

    @property
    def extra_counts_per_division(self):
        r'''Gets extra counts per division.

        Treats overly large and overly small values of
        `extra_counts_per_division` modularly. Denote by
        `unprolated_note_count` the number of unprolated notes included in any
        division (as though `extra_counts_per_division` were set to zero). Then
        the actual number of extra counts included per division is given by two
        formulas:

        * The actual number of extra counts included per division is given by
          ``extra_counts_per_division % unprolated_note_count`` when
          `extra_counts_per_division` is positive.

        * The actual number of extra counts included per division is given by
          the formula
          ``extra_counts_per_division % ceiling(unprolated_note_count / 2)``
          when `extra_counts_per_division` is negative.

        These formulas ensure that:

        * even very large and very small values of
          `extra_counts_per_division` produce valid output, and that

        * the values given as the rhythm-maker's `denominators` are always
          respected. A very large value of `extra_counts_per_division`, for
          example, never causes a `16`-denominated division to result 32nd or
          64th note rhythms; `16`-denominated divisions always produce 16th
          note rhythms.

        ..  container:: example

            Four missing counts per division:

            ::


                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[16],
                ...     extra_counts_per_division=[-4],
                ...     )

            ::

                >>> divisions = [(1, 16), (2, 16), (3, 16), (4, 16), (5, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 1/16
                        {
                            c'16
                        }
                    }
                    {
                        \time 2/16
                        {
                            c'16 [
                            c'16 ]
                        }
                    }
                    {
                        \time 3/16
                        {
                            c'16 [
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 4/16
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 5/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 5/4 {
                            c'16 [
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                }

        ..  container:: example

            Three missing counts per division:

            ::


                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[16],
                ...     extra_counts_per_division=[-3],
                ...     )

            ::

                >>> divisions = [(1, 16), (2, 16), (3, 16), (4, 16), (5, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 1/16
                        {
                            c'16
                        }
                    }
                    {
                        \time 2/16
                        {
                            c'16 [
                            c'16 ]
                        }
                    }
                    {
                        \time 3/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/2 {
                            c'16 [
                            c'16 ]
                        }
                    }
                    {
                        \time 4/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 4/3 {
                            c'16 [
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 5/16
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                }

        ..  container:: example

            Two missing counts per division:

            ::


                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[16],
                ...     extra_counts_per_division=[-2],
                ...     )

            ::

                >>> divisions = [(1, 16), (2, 16), (3, 16), (4, 16), (5, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 1/16
                        {
                            c'16
                        }
                    }
                    {
                        \time 2/16
                        {
                            c'16 [
                            c'16 ]
                        }
                    }
                    {
                        \time 3/16
                        {
                            c'16 [
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 4/16
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 5/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 5/3 {
                            c'16 [
                            c'16
                            c'16 ]
                        }
                    }
                }

        ..  container:: example

            One missing count per division:

            ::


                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[16],
                ...     extra_counts_per_division=[-1],
                ...     )

            ::

                >>> divisions = [(1, 16), (2, 16), (3, 16), (4, 16), (5, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 1/16
                        {
                            c'16
                        }
                    }
                    {
                        \time 2/16
                        {
                            c'16 [
                            c'16 ]
                        }
                    }
                    {
                        \time 3/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/2 {
                            c'16 [
                            c'16 ]
                        }
                    }
                    {
                        \time 4/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 4/3 {
                            c'16 [
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 5/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 5/4 {
                            c'16 [
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                }

        ..  container:: example

            Neither missing nor extra counts per division:

            ::


                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[16],
                ...     extra_counts_per_division=None,
                ...     )

            ::

                >>> divisions = [(1, 16), (2, 16), (3, 16), (4, 16), (5, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 1/16
                        {
                            c'16
                        }
                    }
                    {
                        \time 2/16
                        {
                            c'16 [
                            c'16 ]
                        }
                    }
                    {
                        \time 3/16
                        {
                            c'16 [
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 4/16
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 5/16
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                }

        ..  container:: example

            One extra count per division:

            ::


                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[16],
                ...     extra_counts_per_division=[1],
                ...     )

            ::

                >>> divisions = [(1, 16), (2, 16), (3, 16), (4, 16), (5, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 1/16
                        {
                            c'16
                        }
                    }
                    {
                        \time 2/16
                        \times 2/3 {
                            c'16 [
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 3/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/4 {
                            c'16 [
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 4/16
                        \times 4/5 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 5/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 5/6 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                }

        ..  container:: example

            Two extra counts per division:

            ::


                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[16],
                ...     extra_counts_per_division=[2],
                ...     )

            ::

                >>> divisions = [(1, 16), (2, 16), (3, 16), (4, 16), (5, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 1/16
                        {
                            c'16
                        }
                    }
                    {
                        \time 2/16
                        {
                            c'16 [
                            c'16 ]
                        }
                    }
                    {
                        \time 3/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/5 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 4/16
                        \times 4/6 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 5/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 5/7 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                }

        ..  container:: example

            Three extra counts per division:

            ::


                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[16],
                ...     extra_counts_per_division=[3],
                ...     )

            ::

                >>> divisions = [(1, 16), (2, 16), (3, 16), (4, 16), (5, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 1/16
                        {
                            c'16
                        }
                    }
                    {
                        \time 2/16
                        \times 2/3 {
                            c'16 [
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 3/16
                        {
                            c'16 [
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 4/16
                        \times 4/7 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 5/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 5/8 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                }


        ..  container:: example

            Four extra counts per division:

            ::


                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[16],
                ...     extra_counts_per_division=[4],
                ...     )

            ::

                >>> divisions = [(1, 16), (2, 16), (3, 16), (4, 16), (5, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 1/16
                        {
                            c'16
                        }
                    }
                    {
                        \time 2/16
                        {
                            c'16 [
                            c'16 ]
                        }
                    }
                    {
                        \time 3/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/4 {
                            c'16 [
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 4/16
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 5/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 5/9 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                }

        Returns (possibly empty) tuple of integers or none.
        '''
        if self._extra_counts_per_division:
            return list(self._extra_counts_per_division)

    @property
    def logical_tie_masks(self):
        r'''Gets logical tie masks.

        ..  container:: example

            No logical tie masks:

            ::

                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker()

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                }

        ..  container:: example

            Silences every third logical tie:

            ::

                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     logical_tie_masks=[
                ...         rhythmmakertools.silence_every([0], period=3),
                ...         ],
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        {
                            r8
                            c'8 [
                            c'8 ]
                            r8
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'8 [
                            c'8 ]
                            r8
                        }
                    }
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8 ]
                            r8
                            c'8
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'8
                            r8
                            c'8
                        }
                    }
                }

        ..  container:: example

            Silences every logical tie except the first two and last two:

            ::

                >>> pattern_1 = patterntools.select_all()
                >>> pattern_2 = patterntools.select_first(2)
                >>> pattern_3 = patterntools.select_last(2)
                >>> pattern = pattern_1 ^ pattern_2 ^ pattern_3
                >>> mask = rhythmmakertools.silence(pattern)
                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     logical_tie_masks=mask,
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8 ]
                            r8
                            r8
                        }
                    }
                    {
                        \time 3/8
                        {
                            r8
                            r8
                            r8
                        }
                    }
                    {
                        \time 4/8
                        {
                            r8
                            r8
                            r8
                            r8
                        }
                    }
                    {
                        \time 3/8
                        {
                            r8
                            c'8 [
                            c'8 ]
                        }
                    }
                }

        ..  container:: example

            With ties across divisions:

            ::

                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     tie_specifier=rhythmmakertools.TieSpecifier(
                ...         tie_across_divisions=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ~ ]
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'8 [
                            c'8
                            c'8 ~ ]
                        }
                    }
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8 ~ ]
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                }

            Silences every fourth logical tie:

            ::

                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     logical_tie_masks=rhythmmakertools.silence_every(
                ...         indices=[3],
                ...         period=4,
                ...         ),
                ...     tie_specifier=rhythmmakertools.TieSpecifier(
                ...         tie_across_divisions=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8
                            c'8 ]
                            r8
                        }
                    }
                    {
                        \time 3/8
                        {
                            r8
                            c'8 [
                            c'8 ~ ]
                        }
                    }
                    {
                        \time 4/8
                        {
                            c'8 [
                            c'8 ]
                            r8
                            c'8 ~
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'8 [
                            c'8
                            c'8 ]
                        }
                    }
                }

            Silencing the fourth logical tie produces two rests. Silencing the
            eighth logical tie produces only one rest.

        Returns patterns or none.
        '''
        superclass = super(EvenDivisionRhythmMaker, self)
        return superclass.logical_tie_masks

    @property
    def preferred_denominator(self):
        r'''Gets preferred denominator.

        ..  container:: example

            No preferred denominator:

            ::

                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[16],
                ...     extra_counts_per_division=[4],
                ...     preferred_denominator=None,
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        \times 2/3 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 3/8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/5 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 4/8
                        \times 2/3 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 3/8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/5 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                }

            Expresses tuplet ratios in the usual way with numerator and
            denominator relatively prime.

        ..  container:: example

            Preferred denominator equal to 4:

            ::

                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[16],
                ...     extra_counts_per_division=[4],
                ...     preferred_denominator=4,
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        \times 4/6 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 3/8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/5 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 4/8
                        \times 4/6 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 3/8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/5 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                }

            Preferred denominator equal to 8:

            ::

                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[16],
                ...     extra_counts_per_division=[4],
                ...     preferred_denominator=8,
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        \times 8/12 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 3/8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/5 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 4/8
                        \times 8/12 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 3/8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/5 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                }

            Preferred denominator equal to 16:

            ::

                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[16],
                ...     extra_counts_per_division=[4],
                ...     preferred_denominator=16,
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        \times 16/24 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 3/8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/5 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 4/8
                        \times 16/24 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 3/8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/5 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                }

        ..  container:: example

            Preferred denominator taken from count of elements in tuplet:

            ::

                >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
                ...     denominators=[16],
                ...     extra_counts_per_division=[4],
                ...     preferred_denominator='from_counts',
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        \times 8/12 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 3/8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 6/10 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 4/8
                        \times 8/12 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 3/8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 6/10 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                }

        Defaults to none.

        Set to none or positive integer.

        Returns none or positive integer.
        '''
        return self._preferred_denominator

    @property
    def tuplet_spelling_specifier(self):
        r'''Gets tuplet spelling specifier.

        ..  note:: not yet implemented.

        Returns tuplet spelling specifier or none.
        '''
        superclass = super(EvenDivisionRhythmMaker, self)
        return superclass.tuplet_spelling_specifier



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__contexts__Voice_01():

    target = Voice([])

    assert format(target) == stringtools.normalize(
        r'''
        \new Voice {
        }
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result



# -*- coding: utf-8 -*-
import copy
from abjad.tools import durationtools
from abjad.tools import markuptools
from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools.topleveltools import new
from abjad.tools.mathtools.BoundedObject import BoundedObject


class Timespan(BoundedObject):
    r'''A timespan.

    ..  container:: example

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 12)
            >>> timespan_3 = timespantools.Timespan(-2, 2)
            >>> timespan_4 = timespantools.Timespan(10, 20)

    Timespans are closed-open intervals.

    Timespans are immutable.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Timespans'

    __slots__ = (
        '_start_offset',
        '_stop_offset',
        )

    ### INITIALIZER ###

    def __init__(self, start_offset=NegativeInfinity, stop_offset=Infinity):
        BoundedObject.__init__(self)
        if start_offset is None:
            start_offset = NegativeInfinity
        if stop_offset is None:
            stop_offset = Infinity
        start_offset = self._initialize_offset(start_offset)
        stop_offset = self._initialize_offset(stop_offset)
        assert start_offset <= stop_offset, repr((start_offset, stop_offset))
        self._start_offset = start_offset
        self._stop_offset = stop_offset

    ### SPECIAL METHODS ###

    def __and__(self, argument):
        r'''Logical AND of two timespans.

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 12)
            >>> timespan_3 = timespantools.Timespan(-2, 2)
            >>> timespan_4 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1 & timespan_2
            TimespanList([Timespan(start_offset=Offset(5, 1), stop_offset=Offset(10, 1))])

        ::

            >>> timespan_1 & timespan_3
            TimespanList([Timespan(start_offset=Offset(0, 1), stop_offset=Offset(2, 1))])

        ::

            >>> timespan_1 & timespan_4
            TimespanList([])

        ::

            >>> timespan_2 & timespan_3
            TimespanList([])

        ::

            >>> timespan_2 & timespan_4
            TimespanList([Timespan(start_offset=Offset(10, 1), stop_offset=Offset(12, 1))])

        ::

            >>> timespan_3 & timespan_4
            TimespanList([])

        Returns timespan list.
        '''
        from abjad.tools import timespantools
        from abjad.tools.topleveltools import new
        argument = self._get_timespan(argument)
        if not self.intersects_timespan(argument):
            return timespantools.TimespanList()
        new_start_offset = max(self._start_offset, argument.start_offset)
        new_stop_offset = min(self._stop_offset, argument.stop_offset)
        timespan = new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return timespantools.TimespanList([timespan])

    def __eq__(self, timespan):
        r'''Is true when `timespan` is a timespan with equal offsets.

        ::

            >>> timespantools.Timespan(1, 3) == timespantools.Timespan(1, 3)
            True

        Otherwise false:

        ::

            >>> timespantools.Timespan(1, 3) == timespantools.Timespan(2, 3)
            False

        Returns true or false.
        '''
        from abjad.tools import systemtools
        return systemtools.TestManager.compare_objects(self, timespan)

    def __format__(self, format_specification=''):
        r'''Formats timespan.

        Set `format_specification` to `''` or `'storage'`.

        ::

            >>> print(format(timespan_1))
            timespantools.Timespan(
                start_offset=durationtools.Offset(0, 1),
                stop_offset=durationtools.Offset(10, 1),
                )

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __ge__(self, argument):
        r'''Is true when `argument` start offset is greater or equal
        to timespan start offset.

        ::

            >>> timespan_2 >= timespan_3
            True

        Otherwise false:

        ::

            >>> timespan_1 >= timespan_2
            False

        Returns true or false.
        '''
        expr_start_offset, expr_stop_offset = \
            self._get_start_offset_and_maybe_stop_offset(argument)
        if expr_stop_offset is not None:
            if self._start_offset >= expr_start_offset:
                return True
            elif self._start_offset == expr_start_offset and \
                self._stop_offset >= expr_stop_offset:
                return True
            return False
        return self._start_offset >= expr_start_offset

    def __gt__(self, argument):
        r'''Is true when `argument` start offset is greater than
        timespan start offset.

        ::

            >>> timespan_2 > timespan_3
            True

        Otherwise false:

        ::

            >>> timespan_1 > timespan_2
            False

        Returns true or false.
        '''
        expr_start_offset, expr_stop_offset = \
            self._get_start_offset_and_maybe_stop_offset(argument)
        if expr_stop_offset is not None:
            if self._start_offset > expr_start_offset:
                return True
            elif self._start_offset == expr_start_offset and \
                self._stop_offset > expr_stop_offset:
                return True
            return False
        return self._start_offset > expr_start_offset

    def __hash__(self):
        r'''Hashes timespan.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        from abjad.tools import systemtools
        hash_values = systemtools.StorageFormatAgent(self).get_hash_values()
        return hash(hash_values)

    def __illustrate__(self, range_=None, scale=None):
        r'''Illustrates timespan.

        Returns LilyPond file.
        '''
        import abjad
        timespans = abjad.timespantools.TimespanList([self])
        return timespans.__illustrate__(
            range_=range_,
            scale=scale,
            )

    def __le__(self, argument):
        r'''Is true when `argument` start offset is less than or equal to
        timespan start offset.

        ::

            >>> timespan_2 <= timespan_3
            False

        Otherwise false:

        ::

            >>> timespan_1 <= timespan_2
            True

        Returns true or false.
        '''
        expr_start_offset, expr_stop_offset = \
            self._get_start_offset_and_maybe_stop_offset(argument)
        if expr_stop_offset is not None:
            if self._start_offset <= expr_start_offset:
                return True
            elif self._start_offset == expr_start_offset and \
                self._stop_offset <= expr_stop_offset:
                return True
            return False
        return self._start_offset <= expr_start_offset

    def __len__(self):
        r'''Defined equal to ``1`` for all timespans.

        ::

            >>> len(timespan_1)
            1

        Returns positive integer.
        '''
        return 1

    def __lt__(self, argument):
        r'''Is true when `argument` start offset is less than timespan start offset.

        ::

            >>> timespan_1 < timespan_2
            True

        Otherwise false:

        ::

            >>> timespan_2 < timespan_3
            False

        Returns true or false.
        '''
        expr_start_offset, expr_stop_offset = \
            self._get_start_offset_and_maybe_stop_offset(argument)
        if expr_stop_offset is not None:
            if self._start_offset < expr_start_offset:
                return True
            elif self._start_offset == expr_start_offset and \
                self._stop_offset < expr_stop_offset:
                return True
            return False
        return self._start_offset < expr_start_offset

    def __ne__(self, timespan):
        r'''Is true when `timespan` is not a timespan with equivalent offsets.

        ::

            >>> timespantools.Timespan(1, 3) != timespantools.Timespan(2, 3)
            True

        Otherwise false:

        ::

            >>> timespantools.Timespan(1, 3) != timespantools.Timespan(2/2, (3, 1))
            False

        Returns true or false.
        '''
        return not self == timespan

    def __or__(self, argument):
        r'''Logical OR of two timespans.

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 12)
            >>> timespan_3 = timespantools.Timespan(-2, 2)
            >>> timespan_4 = timespantools.Timespan(10, 20)

        ::

            >>> new_timespan = timespan_1 | timespan_2
            >>> print(format(new_timespan))
            timespantools.TimespanList(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(0, 1),
                        stop_offset=durationtools.Offset(12, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_1 | timespan_3
            >>> print(format(new_timespan))
            timespantools.TimespanList(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(-2, 1),
                        stop_offset=durationtools.Offset(10, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_1 | timespan_4
            >>> print(format(new_timespan))
            timespantools.TimespanList(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(0, 1),
                        stop_offset=durationtools.Offset(20, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_2 | timespan_3
            >>> print(format(new_timespan))
            timespantools.TimespanList(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(-2, 1),
                        stop_offset=durationtools.Offset(2, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(5, 1),
                        stop_offset=durationtools.Offset(12, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_2 | timespan_4
            >>> print(format(new_timespan))
            timespantools.TimespanList(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(5, 1),
                        stop_offset=durationtools.Offset(20, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_3 | timespan_4
            >>> print(format(new_timespan))
            timespantools.TimespanList(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(-2, 1),
                        stop_offset=durationtools.Offset(2, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(10, 1),
                        stop_offset=durationtools.Offset(20, 1),
                        ),
                    ]
                )

        Returns timespan list.
        '''
        from abjad.tools import timespantools
        argument = self._get_timespan(argument)
        if not self.intersects_timespan(argument) and \
            not self.is_tangent_to_timespan(argument):
            result = timespantools.TimespanList([self, argument])
            result.sort()
            return result
        new_start_offset = min(self._start_offset, argument.start_offset)
        new_stop_offset = max(self._stop_offset, argument.stop_offset)
        timespan = new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return timespantools.TimespanList([timespan])

    def __sub__(self, argument):
        r'''Subtract `argument` from timespan.

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 12)
            >>> timespan_3 = timespantools.Timespan(-2, 2)
            >>> timespan_4 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1 - timespan_1
            TimespanList([])

        ::

            >>> timespan_1 - timespan_2
            TimespanList([Timespan(start_offset=Offset(0, 1), stop_offset=Offset(5, 1))])

        ::

            >>> timespan_1 - timespan_3
            TimespanList([Timespan(start_offset=Offset(2, 1), stop_offset=Offset(10, 1))])

        ::

            >>> timespan_1 - timespan_4
            TimespanList([Timespan(start_offset=Offset(0, 1), stop_offset=Offset(10, 1))])

        ::

            >>> timespan_2 - timespan_1
            TimespanList([Timespan(start_offset=Offset(10, 1), stop_offset=Offset(12, 1))])

        ::

            >>> timespan_2 - timespan_2
            TimespanList([])

        ::

            >>> timespan_2 - timespan_3
            TimespanList([Timespan(start_offset=Offset(5, 1), stop_offset=Offset(12, 1))])

        ::

            >>> timespan_2 - timespan_4
            TimespanList([Timespan(start_offset=Offset(5, 1), stop_offset=Offset(10, 1))])

        ::

            >>> timespan_3 - timespan_3
            TimespanList([])

        ::

            >>> timespan_3 - timespan_1
            TimespanList([Timespan(start_offset=Offset(-2, 1), stop_offset=Offset(0, 1))])

        ::

            >>> timespan_3 - timespan_2
            TimespanList([Timespan(start_offset=Offset(-2, 1), stop_offset=Offset(2, 1))])

        ::

            >>> timespan_3 - timespan_4
            TimespanList([Timespan(start_offset=Offset(-2, 1), stop_offset=Offset(2, 1))])

        ::

            >>> timespan_4 - timespan_4
            TimespanList([])

        ::

            >>> timespan_4 - timespan_1
            TimespanList([Timespan(start_offset=Offset(10, 1), stop_offset=Offset(20, 1))])

        ::

            >>> timespan_4 - timespan_2
            TimespanList([Timespan(start_offset=Offset(12, 1), stop_offset=Offset(20, 1))])

        ::

            >>> timespan_4 - timespan_3
            TimespanList([Timespan(start_offset=Offset(10, 1), stop_offset=Offset(20, 1))])

        Returns timespan list.
        '''
        from abjad.tools import timespantools
        argument = self._get_timespan(argument)
        timespans = timespantools.TimespanList()
        if not self.intersects_timespan(argument):
            timespans.append(copy.deepcopy(self))
        elif argument.trisects_timespan(self):
            new_start_offset = self._start_offset
            new_stop_offset = argument.start_offset
            timespan = new(
                self,
                start_offset=new_start_offset,
                stop_offset=new_stop_offset,
                )
            timespans.append(timespan)
            new_start_offset = argument.stop_offset
            new_stop_offset = self._stop_offset
            timespan = new(
                self,
                start_offset=new_start_offset,
                stop_offset=new_stop_offset,
                )
            timespans.append(timespan)
        elif argument.contains_timespan_improperly(self):
            pass
        elif argument.overlaps_only_start_of_timespan(self):
            new_start_offset = argument.stop_offset
            new_stop_offset = self._stop_offset
            timespan = new(
                self,
                start_offset=new_start_offset,
                stop_offset=new_stop_offset,
                )
            timespans.append(timespan)
        elif argument.overlaps_only_stop_of_timespan(self):
            new_start_offset = self._start_offset
            new_stop_offset = argument.start_offset
            timespan = new(
                self,
                start_offset=new_start_offset,
                stop_offset=new_stop_offset,
                )
            timespans.append(timespan)
        elif argument.starts_when_timespan_starts(self) and \
            argument.stops_before_timespan_stops(self):
            new_start_offset = argument.stop_offset
            new_stop_offset = self._stop_offset
            timespan = new(
                self,
                start_offset=new_start_offset,
                stop_offset=new_stop_offset,
                )
            timespans.append(timespan)
        elif argument.stops_when_timespan_stops(self) and \
            argument.starts_after_timespan_starts(self):
            new_start_offset = self._start_offset
            new_stop_offset = argument.start_offset
            timespan = new(
                self,
                start_offset=new_start_offset,
                stop_offset=new_stop_offset,
                )
            timespans.append(timespan)
        else:
            raise ValueError(self, argument)
        return timespans

    def __xor__(self, argument):
        r'''Logical XOR of two timespans.

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 12)
            >>> timespan_3 = timespantools.Timespan(-2, 2)
            >>> timespan_4 = timespantools.Timespan(10, 20)

        ::

            >>> new_timespan = timespan_1 ^ timespan_2
            >>> print(format(new_timespan))
            timespantools.TimespanList(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(0, 1),
                        stop_offset=durationtools.Offset(5, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(10, 1),
                        stop_offset=durationtools.Offset(12, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_1 ^ timespan_3
            >>> print(format(new_timespan))
            timespantools.TimespanList(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(-2, 1),
                        stop_offset=durationtools.Offset(0, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(2, 1),
                        stop_offset=durationtools.Offset(10, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_1 ^ timespan_4
            >>> print(format(new_timespan))
            timespantools.TimespanList(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(0, 1),
                        stop_offset=durationtools.Offset(10, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(10, 1),
                        stop_offset=durationtools.Offset(20, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_2 ^ timespan_3
            >>> print(format(new_timespan))
            timespantools.TimespanList(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(-2, 1),
                        stop_offset=durationtools.Offset(2, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(5, 1),
                        stop_offset=durationtools.Offset(12, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_2 ^ timespan_4
            >>> print(format(new_timespan))
            timespantools.TimespanList(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(5, 1),
                        stop_offset=durationtools.Offset(10, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(12, 1),
                        stop_offset=durationtools.Offset(20, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_3 ^ timespan_4
            >>> print(format(new_timespan))
            timespantools.TimespanList(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(-2, 1),
                        stop_offset=durationtools.Offset(2, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(10, 1),
                        stop_offset=durationtools.Offset(20, 1),
                        ),
                    ]
                )

        Returns timespan list.
        '''
        from abjad.tools import timespantools
        argument = self._get_timespan(argument)
        if not self.intersects_timespan(argument) or \
            self.is_tangent_to_timespan(argument):
            result = timespantools.TimespanList()
            result.append(copy.deepcopy(self))
            result.append(copy.deepcopy(argument))
            result.sort()
            return result
        result = timespantools.TimespanList()
        start_offsets = [self._start_offset, argument.start_offset]
        stop_offsets = [self._stop_offset, argument.stop_offset]
        start_offsets.sort()
        stop_offsets.sort()
        timespan_1 = new(
            self,
            start_offset=start_offsets[0],
            stop_offset=start_offsets[1],
            )
        timespan_2 = new(
            self,
            start_offset=stop_offsets[0],
            stop_offset=stop_offsets[1],
            )
        if timespan_1.is_well_formed:
            result.append(timespan_1)
        if timespan_2.is_well_formed:
            result.append(timespan_2)
        result.sort()
        return result

    ### PRIVATE METHODS ###

    def _as_postscript(
        self,
        postscript_x_offset,
        postscript_y_offset,
        postscript_scale,
        ):
        start = (float(self._start_offset) * postscript_scale)
        start -= postscript_x_offset
        stop = (float(self._stop_offset) * postscript_scale)
        stop -= postscript_x_offset
        ps = markuptools.Postscript()
        ps = ps.moveto(start, postscript_y_offset)
        ps = ps.lineto(stop, postscript_y_offset)
        ps = ps.stroke()
        ps = ps.moveto(start, postscript_y_offset + 0.75)
        ps = ps.lineto(start, postscript_y_offset - 0.75)
        ps = ps.stroke()
        ps = ps.moveto(stop, postscript_y_offset + 0.75)
        ps = ps.lineto(stop, postscript_y_offset - 0.75)
        ps = ps.stroke()
        return ps

    def _can_fuse(self, argument):
        if isinstance(argument, type(self)):
            return self.intersects_timespan(argument) or \
                self.stops_when_timespan_starts(argument)
        return False

    @staticmethod
    def _get_offsets(argument):
        if isinstance(argument, Timespan):
            pass
        elif hasattr(argument, 'timespan'):
            argument = argument.timespan
        elif hasattr(argument, '_get_timespan'):
            argument = argument._get_timespan()
        else:
            raise ValueError(argument)
        return argument._start_offset, argument._stop_offset

    @staticmethod
    def _get_start_offset_and_maybe_stop_offset(argument):
        if isinstance(argument, Timespan):
            pass
        elif hasattr(argument, 'timespan'):
            argument = argument.timespan
        elif hasattr(argument, '_get_timespan'):
            argument = argument._get_timespan()
        start_offset = getattr(argument, 'start_offset', None)
        if start_offset is None:
            raise ValueError(argument)
        stop_offset = getattr(argument, 'stop_offset', None)
        return start_offset, stop_offset

    def _get_timespan(self, argument):
        if isinstance(argument, Timespan):
            start_offset, stop_offset = argument.offsets
        elif hasattr(argument, 'timespan'):
            start_offset, stop_offset = argument.timespan.offsets
        elif hasattr(argument, '_get_timespan'):
            start_offset, stop_offset = argument._get_timespan().offsets
        # TODO: remove this branch in favor of the _get_timespan above
        #elif hasattr(argument, 'get_timespan'):
        #    start_offset, stop_offset = argument.get_timespan().offsets
        else:
            raise ValueError(argument)
        return new(
            self,
            start_offset=start_offset,
            stop_offset=stop_offset,
            )

    def _implements_timespan_interface(self, timespan):
        if hasattr(timespan, 'start_offset') and \
            hasattr(timespan, 'stop_offset'):
            return True
        if hasattr(timespan, '_get_timespan'):
            return True
        # TODO: remove this branch in favor of the _get_timespan above
        if hasattr(timespan, 'get_timespan'):
            return True
        if hasattr(timespan, 'timespan'):
            return True
        return False

    def _initialize_offset(self, offset):
        if offset in (NegativeInfinity, Infinity):
            return offset
        return durationtools.Offset(offset)

    ### PUBLIC METHODS ###

    def contains_timespan_improperly(self, timespan):
        r'''Is true when timespan contains `timespan` improperly.
        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 10)

        ::

            >>> timespan_1.contains_timespan_improperly(timespan_1)
            True
            >>> timespan_1.contains_timespan_improperly(timespan_2)
            True
            >>> timespan_2.contains_timespan_improperly(timespan_1)
            False
            >>> timespan_2.contains_timespan_improperly(timespan_2)
            True

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            self_start_offset <= expr_start_offset and
            expr_stop_offset <= self_stop_offset
            )

    def curtails_timespan(self, timespan):
        r'''Is true when timespan curtails `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 10)

        ::

            >>> timespan_1.curtails_timespan(timespan_1)
            False
            >>> timespan_1.curtails_timespan(timespan_2)
            False
            >>> timespan_2.curtails_timespan(timespan_1)
            True
            >>> timespan_2.curtails_timespan(timespan_2)
            False

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            expr_start_offset < self_start_offset and
            self_start_offset <= expr_stop_offset and
            expr_stop_offset <= self_stop_offset
            )

    def delays_timespan(self, timespan):
        r'''Is true when timespan delays `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)
            >>> timespan_3 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.delays_timespan(timespan_2)
            True
            >>> timespan_2.delays_timespan(timespan_3)
            True

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            self_start_offset <= expr_start_offset and
            expr_start_offset < self_stop_offset
            )

    def divide_by_ratio(self, ratio):
        r'''Divides timespan by `ratio`.

        ::

            >>> timespan = timespantools.Timespan((1, 2), (3, 2))

        ::

            >>> for x in timespan.divide_by_ratio((1, 2, 1)):
            ...     x
            Timespan(start_offset=Offset(1, 2), stop_offset=Offset(3, 4))
            Timespan(start_offset=Offset(3, 4), stop_offset=Offset(5, 4))
            Timespan(start_offset=Offset(5, 4), stop_offset=Offset(3, 2))

        Returns tuple of newly constructed timespans.
        '''
        if isinstance(ratio, int):
            ratio = ratio * (1, )
        ratio = mathtools.Ratio(ratio)
        unit_duration = self.duration / sum(ratio.numbers)
        part_durations = [
            numerator * unit_duration for numerator in ratio.numbers
            ]
        start_offsets = mathtools.cumulative_sums(
            [self._start_offset] + part_durations,
            start=None,
            )
        offset_pairs = sequencetools.Sequence(start_offsets).nwise()
        result = [type(self)(*offset_pair) for offset_pair in offset_pairs]
        return tuple(result)

    def get_overlap_with_timespan(self, timespan):
        '''Gets duration of overlap with `timespan`.

        ::

            >>> timespan_1 = timespantools.Timespan(0, 15)
            >>> timespan_2 = timespantools.Timespan(5, 10)
            >>> timespan_3 = timespantools.Timespan(6, 6)
            >>> timespan_4 = timespantools.Timespan(12, 22)


        ::

            >>> timespan_1.get_overlap_with_timespan(timespan_1)
            Duration(15, 1)

        ::

            >>> timespan_1.get_overlap_with_timespan(timespan_2)
            Duration(5, 1)

        ::

            >>> timespan_1.get_overlap_with_timespan(timespan_3)
            Duration(0, 1)

        ::

            >>> timespan_1.get_overlap_with_timespan(timespan_4)
            Duration(3, 1)

        ::

            >>> timespan_2.get_overlap_with_timespan(timespan_2)
            Duration(5, 1)

        ::

            >>> timespan_2.get_overlap_with_timespan(timespan_3)
            Duration(0, 1)

        ::

            >>> timespan_2.get_overlap_with_timespan(timespan_4)
            Duration(0, 1)

        ::

            >>> timespan_3.get_overlap_with_timespan(timespan_3)
            Duration(0, 1)

        ::

            >>> timespan_3.get_overlap_with_timespan(timespan_4)
            Duration(0, 1)

        ::

            >>> timespan_4.get_overlap_with_timespan(timespan_4)
            Duration(10, 1)


        Returns duration.
        '''
        if self._implements_timespan_interface(timespan):
            result = durationtools.Duration(
                sum(x.duration for x in self & timespan))
            return result

    def happens_during_timespan(self, timespan):
        r'''Is true when timespan happens during `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 10)

        ::

            >>> timespan_1.happens_during_timespan(timespan_1)
            True
            >>> timespan_1.happens_during_timespan(timespan_2)
            False
            >>> timespan_2.happens_during_timespan(timespan_1)
            True
            >>> timespan_2.happens_during_timespan(timespan_2)
            True

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            expr_start_offset <= self_start_offset and
            self_stop_offset <= expr_stop_offset
            )

    def intersects_timespan(self, timespan):
        r'''Is true when timespan intersects `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)
            >>> timespan_3 = timespantools.Timespan(10, 15)

        ::

            >>> timespan_1.intersects_timespan(timespan_1)
            True
            >>> timespan_1.intersects_timespan(timespan_2)
            True
            >>> timespan_1.intersects_timespan(timespan_3)
            False

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            (
                expr_start_offset <= self_start_offset and
                self_start_offset < expr_stop_offset
            ) or (
                self_start_offset <= expr_start_offset and
                expr_start_offset < self_stop_offset
                )
            )

    def is_congruent_to_timespan(self, timespan):
        r'''Is true when timespan is congruent to `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.is_congruent_to_timespan(timespan_1)
            True
            >>> timespan_1.is_congruent_to_timespan(timespan_2)
            False
            >>> timespan_2.is_congruent_to_timespan(timespan_1)
            False
            >>> timespan_2.is_congruent_to_timespan(timespan_2)
            True

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            expr_start_offset == self_start_offset and
            expr_stop_offset == self_stop_offset
            )

    def is_tangent_to_timespan(self, timespan):
        r'''Is true when timespan is tangent to `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.is_tangent_to_timespan(timespan_1)
            False
            >>> timespan_1.is_tangent_to_timespan(timespan_2)
            True
            >>> timespan_2.is_tangent_to_timespan(timespan_1)
            True
            >>> timespan_2.is_tangent_to_timespan(timespan_2)
            False

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            self_stop_offset == expr_start_offset or
            expr_stop_offset == self_start_offset
            )

    def overlaps_all_of_timespan(self, timespan):
        r'''Is true when timespan overlaps all of `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 6)
            >>> timespan_3 = timespantools.Timespan(5, 10)

        ::

            >>> timespan_1.overlaps_all_of_timespan(timespan_1)
            False
            >>> timespan_1.overlaps_all_of_timespan(timespan_2)
            True
            >>> timespan_1.overlaps_all_of_timespan(timespan_3)
            False

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            self_start_offset < expr_start_offset and
            expr_stop_offset < self_stop_offset
            )

    def overlaps_only_start_of_timespan(self, timespan):
        r'''Is true when timespan overlaps only start of `timespan`.
        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(-5, 5)
            >>> timespan_3 = timespantools.Timespan(4, 6)
            >>> timespan_4 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.overlaps_only_start_of_timespan(timespan_1)
            False
            >>> timespan_1.overlaps_only_start_of_timespan(timespan_2)
            False
            >>> timespan_1.overlaps_only_start_of_timespan(timespan_3)
            False
            >>> timespan_1.overlaps_only_start_of_timespan(timespan_4)
            True

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            self_start_offset < expr_start_offset and
            expr_start_offset < self_stop_offset and
            self_stop_offset <= expr_stop_offset
            )

    def overlaps_only_stop_of_timespan(self, timespan):
        r'''Is true when timespan overlaps only stop of `timespan`.
        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(-5, 5)
            >>> timespan_3 = timespantools.Timespan(4, 6)
            >>> timespan_4 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.overlaps_only_stop_of_timespan(timespan_1)
            False
            >>> timespan_1.overlaps_only_stop_of_timespan(timespan_2)
            True
            >>> timespan_1.overlaps_only_stop_of_timespan(timespan_3)
            False
            >>> timespan_1.overlaps_only_stop_of_timespan(timespan_4)
            False

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            expr_start_offset <= self_start_offset and
            self_start_offset < expr_stop_offset and
            expr_stop_offset < self_stop_offset
            )

    def overlaps_start_of_timespan(self, timespan):
        r'''Is true when timespan overlaps start of `timespan`.
        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(-5, 5)
            >>> timespan_3 = timespantools.Timespan(4, 6)
            >>> timespan_4 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.overlaps_start_of_timespan(timespan_1)
            False
            >>> timespan_1.overlaps_start_of_timespan(timespan_2)
            False
            >>> timespan_1.overlaps_start_of_timespan(timespan_3)
            True
            >>> timespan_1.overlaps_start_of_timespan(timespan_4)
            True

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            self_start_offset < expr_start_offset and
            expr_start_offset < self_stop_offset
            )

    def overlaps_stop_of_timespan(self, timespan):
        r'''Is true when timespan overlaps start of `timespan`.
        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(-5, 5)
            >>> timespan_3 = timespantools.Timespan(4, 6)
            >>> timespan_4 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.overlaps_stop_of_timespan(timespan_1)
            False
            >>> timespan_1.overlaps_stop_of_timespan(timespan_2)
            True
            >>> timespan_1.overlaps_stop_of_timespan(timespan_3)
            True
            >>> timespan_1.overlaps_stop_of_timespan(timespan_4)
            False

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            self_start_offset < expr_stop_offset and
            expr_stop_offset < self_stop_offset
            )

    def reflect(self, axis=None):
        r'''Reflects timespan about `axis`.

        ..  container:: example

            **Example 1.** Reverse timespan about timespan axis:

            ::

                >>> timespantools.Timespan(3, 6).reflect()
                Timespan(start_offset=Offset(3, 1), stop_offset=Offset(6, 1))

        ..  container:: example

            **Example 2.** Reverse timespan about arbitrary axis:

            ::

                >>> timespantools.Timespan(3, 6).reflect(axis=Offset(10))
                Timespan(start_offset=Offset(14, 1), stop_offset=Offset(17, 1))

        Returns new timespan.
        '''
        if axis is None:
            axis = self.axis
        start_distance = self._start_offset - axis
        stop_distance = self._stop_offset - axis
        new_start_offset = axis - stop_distance
        new_stop_offset = axis - start_distance
        return self.set_offsets(new_start_offset, new_stop_offset)

    def round_offsets(self, multiplier, anchor=Left, must_be_well_formed=True):
        '''Rounds timespan offsets to multiple of `multiplier`.

        ::

            >>> timespan = timespantools.Timespan((1, 5), (4, 5))

        ::

            >>> timespan.round_offsets(1)
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(1, 1))

        ::

            >>> timespan.round_offsets(2)
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(2, 1))

        ::

            >>> timespan.round_offsets(
            ...     2,
            ...     anchor=Right,
            ...     )
            Timespan(start_offset=Offset(-2, 1), stop_offset=Offset(0, 1))

        ::

            >>> timespan.round_offsets(
            ...     2,
            ...     anchor=Right,
            ...     must_be_well_formed=False,
            ...     )
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(0, 1))

        Returns new timespan.
        '''
        from abjad.tools.topleveltools import new
        multiplier = abs(durationtools.Multiplier(multiplier))
        assert 0 < multiplier
        new_start_offset = durationtools.Offset(
            int(round(self._start_offset / multiplier)) * multiplier)
        new_stop_offset = durationtools.Offset(
            int(round(self._stop_offset / multiplier)) * multiplier)
        if (new_start_offset == new_stop_offset) and must_be_well_formed:
            if anchor == Left:
                new_stop_offset = new_stop_offset + multiplier
            else:
                new_start_offset = new_start_offset - multiplier
        result = new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return result

    def scale(self, multiplier, anchor=Left):
        r'''Scales timespan by `multiplier`.

            >>> timespan = timespantools.Timespan(3, 6)

        ..  container:: example

            **Example 1.** Scale timespan relative to timespan start offset:

            ::

                >>> timespan.scale(Multiplier(2))
                Timespan(start_offset=Offset(3, 1), stop_offset=Offset(9, 1))

        ..  container:: example

            **Example 2.** Scale timespan relative to timespan stop offset:

            ::

                >>> timespan.scale(Multiplier(2), anchor=Right)
                Timespan(start_offset=Offset(0, 1), stop_offset=Offset(6, 1))

        Returns new timespan.
        '''
        from abjad.tools.topleveltools import new
        multiplier = durationtools.Multiplier(multiplier)
        assert 0 < multiplier
        new_duration = multiplier * self.duration
        if anchor == Left:
            new_start_offset = self._start_offset
            new_stop_offset = self._start_offset + new_duration
        elif anchor == Right:
            new_stop_offset = self._stop_offset
            new_start_offset = self._stop_offset - new_duration
        else:
            raise ValueError
        result = new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return result

    def set_duration(self, duration):
        r'''Sets timespan duration to `duration`.

        ::

            >>> timespan = timespantools.Timespan((1, 2), (3, 2))

        ::

            >>> timespan.set_duration(Duration(3, 5))
            Timespan(start_offset=Offset(1, 2), stop_offset=Offset(11, 10))

        Returns new timespan.
        '''
        from abjad.tools.topleveltools import new
        duration = durationtools.Duration(duration)
        new_stop_offset = self._start_offset + duration
        result = new(
            self,
            stop_offset=new_stop_offset,
            )
        return result

    def set_offsets(self, start_offset=None, stop_offset=None):
        r'''Sets timespan start offset to `start_offset` and
        stop offset to `stop_offset`.

        ::

            >>> timespan = timespantools.Timespan((1, 2), (3, 2))

        ::

            >>> timespan.set_offsets(stop_offset=Offset(7, 8))
            Timespan(start_offset=Offset(1, 2), stop_offset=Offset(7, 8))

        Subtracts negative `start_offset` from existing stop offset:

        ::

            >>> timespan.set_offsets(start_offset=Offset(-1, 2))
            Timespan(start_offset=Offset(1, 1), stop_offset=Offset(3, 2))

        Subtracts negative `stop_offset` from existing stop offset:

        ::

            >>> timespan.set_offsets(stop_offset=Offset(-1, 2))
            Timespan(start_offset=Offset(1, 2), stop_offset=Offset(1, 1))

        Returns new timespan.
        '''
        from abjad.tools.topleveltools import new
        if start_offset is not None and 0 <= start_offset:
            new_start_offset = start_offset
        elif start_offset is not None and start_offset < 0:
            new_start_offset = \
                self._stop_offset + durationtools.Offset(start_offset)
        else:
            new_start_offset = self._start_offset
        if stop_offset is not None and 0 <= stop_offset:
            new_stop_offset = stop_offset
        elif stop_offset is not None and stop_offset < 0:
            new_stop_offset = \
                self._stop_offset + durationtools.Offset(stop_offset)
        else:
            new_stop_offset = self._stop_offset
        result = new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return result

    # TODO: extend to self.split_at_offsets()
    def split_at_offset(self, offset):
        r'''Split into two parts when `offset` happens during timespan:

        ::

            >>> timespan = timespantools.Timespan(0, 5)

        ::

            >>> left, right = timespan.split_at_offset(Offset(2))

        ::

            >>> left
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(2, 1))

        ::

            >>> right
            Timespan(start_offset=Offset(2, 1), stop_offset=Offset(5, 1))

        Otherwise return a copy of timespan:

        ::

            >>> timespan.split_at_offset(Offset(12))[0]
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(5, 1))

        Returns one or two newly constructed timespans.
        '''
        from abjad.tools import timespantools
        offset = durationtools.Offset(offset)
        result = timespantools.TimespanList()
        if self._start_offset < offset < self._stop_offset:
            left = new(
                self,
                start_offset=self._start_offset,
                stop_offset=offset,
                )
            right = new(
                self,
                start_offset=offset,
                stop_offset=self._stop_offset,
                )
            result.append(left)
            result.append(right)
        else:
            result.append(new(self))
        return result

    def split_at_offsets(self, offsets):
        r'''Split into one or more parts when `offsets` happens during
        timespan:

        ::

            >>> timespan = timespantools.Timespan(0, 10)

        ::

            >>> result = timespan.split_at_offsets((1, 3, 7))
            >>> print(format(result))
            timespantools.TimespanList(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(0, 1),
                        stop_offset=durationtools.Offset(1, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(1, 1),
                        stop_offset=durationtools.Offset(3, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(3, 1),
                        stop_offset=durationtools.Offset(7, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(7, 1),
                        stop_offset=durationtools.Offset(10, 1),
                        ),
                    ]
                )

        Otherwise return a timespan list containing a copy of timespan:

        ::

            >>> result = timespan.split_at_offsets((-100,))
            >>> f(result)
            timespantools.TimespanList(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(0, 1),
                        stop_offset=durationtools.Offset(10, 1),
                        ),
                    ]
                )

        Returns one or more newly constructed timespans.
        '''
        from abjad.tools import timespantools
        offsets = [durationtools.Offset(offset) for offset in offsets]
        offsets = [offset for offset in offsets
            if self._start_offset < offset < self._stop_offset]
        offsets = sorted(set(offsets))
        result = timespantools.TimespanList()
        right = new(self)
        for offset in offsets:
            left, right = right.split_at_offset(offset)
            result.append(left)
        result.append(right)
        return result

    def starts_after_offset(self, offset):
        r'''Is true when timespan overlaps start of `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.starts_after_offset(Offset(-5))
            True
            >>> timespan_1.starts_after_offset(Offset(0))
            False
            >>> timespan_1.starts_after_offset(Offset(5))
            False

        Returns true or false.
        '''
        offset = durationtools.Offset(offset)
        return offset < self._start_offset

    def starts_after_timespan_starts(self, timespan):
        r'''Is true when timespan starts after `timespan` starts. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.starts_after_timespan_starts(timespan_1)
            False
            >>> timespan_1.starts_after_timespan_starts(timespan_2)
            False
            >>> timespan_2.starts_after_timespan_starts(timespan_1)
            True
            >>> timespan_2.starts_after_timespan_starts(timespan_2)
            False

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return expr_start_offset < self_start_offset

    def starts_after_timespan_stops(self, timespan):
        r'''Is true when timespan starts after `timespan` stops. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)
            >>> timespan_3 = timespantools.Timespan(10, 20)
            >>> timespan_4 = timespantools.Timespan(15, 25)

        ::

            >>> timespan_1.starts_after_timespan_stops(timespan_1)
            False
            >>> timespan_2.starts_after_timespan_stops(timespan_1)
            False
            >>> timespan_3.starts_after_timespan_stops(timespan_1)
            True
            >>> timespan_4.starts_after_timespan_stops(timespan_1)
            True

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return expr_stop_offset <= self_start_offset

    def starts_at_offset(self, offset):
        r'''Is true when timespan starts at `offset`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.starts_at_offset(Offset(-5))
            False
            >>> timespan_1.starts_at_offset(Offset(0))
            True
            >>> timespan_1.starts_at_offset(Offset(5))
            False

        Returns true or false.
        '''
        offset = durationtools.Offset(offset)
        return self._start_offset == offset

    def starts_at_or_after_offset(self, offset):
        r'''Is true when timespan starts at or after `offset`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.starts_at_or_after_offset(Offset(-5))
            True
            >>> timespan_1.starts_at_or_after_offset(Offset(0))
            True
            >>> timespan_1.starts_at_or_after_offset(Offset(5))
            False

        Returns true or false.
        '''
        offset = durationtools.Offset(offset)
        return offset <= self._start_offset

    def starts_before_offset(self, offset):
        r'''Is true when timespan starts before `offset`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.starts_before_offset(Offset(-5))
            False
            >>> timespan_1.starts_before_offset(Offset(0))
            False
            >>> timespan_1.starts_before_offset(Offset(5))
            True

        Returns true or false.
        '''
        offset = durationtools.Offset(offset)
        return self._start_offset < offset

    def starts_before_or_at_offset(self, offset):
        r'''Is true when timespan starts before or at `offset`.
        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.starts_before_or_at_offset(Offset(-5))
            False
            >>> timespan_1.starts_before_or_at_offset(Offset(0))
            True
            >>> timespan_1.starts_before_or_at_offset(Offset(5))
            True

        Returns true or false.
        '''
        offset = durationtools.Offset(offset)
        return self._start_offset <= offset

    def starts_before_timespan_starts(self, timespan):
        r'''Is true when timespan starts before `timespan` starts.
        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.starts_before_timespan_starts(timespan_1)
            False
            >>> timespan_1.starts_before_timespan_starts(timespan_2)
            True
            >>> timespan_2.starts_before_timespan_starts(timespan_1)
            False
            >>> timespan_2.starts_before_timespan_starts(timespan_2)
            False

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return self_start_offset < expr_start_offset

    def starts_before_timespan_stops(self, timespan):
        r'''Is true when timespan starts before `timespan` stops. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.starts_before_timespan_stops(timespan_1)
            True
            >>> timespan_1.starts_before_timespan_stops(timespan_2)
            True
            >>> timespan_2.starts_before_timespan_stops(timespan_1)
            True
            >>> timespan_2.starts_before_timespan_stops(timespan_2)
            True

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return self_start_offset < expr_stop_offset

    def starts_during_timespan(self, timespan):
        r'''Is true when timespan starts during `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.starts_during_timespan(timespan_1)
            True
            >>> timespan_1.starts_during_timespan(timespan_2)
            False
            >>> timespan_2.starts_during_timespan(timespan_1)
            True
            >>> timespan_2.starts_during_timespan(timespan_2)
            True

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            expr_start_offset <= self_start_offset and
            self_start_offset < expr_stop_offset
            )

    def starts_when_timespan_starts(self, timespan):
        r'''Is true when timespan starts when `timespan` starts. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.starts_when_timespan_starts(timespan_1)
            True
            >>> timespan_1.starts_when_timespan_starts(timespan_2)
            False
            >>> timespan_2.starts_when_timespan_starts(timespan_1)
            False
            >>> timespan_2.starts_when_timespan_starts(timespan_2)
            True

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return expr_start_offset == self_start_offset

    def starts_when_timespan_stops(self, timespan):
        r'''Is true when timespan starts when `timespan` stops. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.starts_when_timespan_stops(timespan_1)
            False
            >>> timespan_1.starts_when_timespan_stops(timespan_2)
            False
            >>> timespan_2.starts_when_timespan_stops(timespan_1)
            True
            >>> timespan_2.starts_when_timespan_stops(timespan_2)
            False

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return self_start_offset == expr_stop_offset

    def stops_after_offset(self, offset):
        r'''Is true when timespan stops after `offset`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.starts_after_offset(Offset(-5))
            True
            >>> timespan_1.starts_after_offset(Offset(0))
            False
            >>> timespan_1.starts_after_offset(Offset(5))
            False

        Returns true or false.
        '''
        offset = durationtools.Offset(offset)
        return offset < self._stop_offset

    def stops_after_timespan_starts(self, timespan):
        r'''Is true when timespan stops when `timespan` starts. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.stops_after_timespan_starts(timespan_1)
            True
            >>> timespan_1.stops_after_timespan_starts(timespan_2)
            False
            >>> timespan_2.stops_after_timespan_starts(timespan_1)
            True
            >>> timespan_2.stops_after_timespan_starts(timespan_2)
            True

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return expr_start_offset < self_stop_offset

    def stops_after_timespan_stops(self, timespan):
        r'''Is true when timespan stops when `timespan` stops. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.stops_after_timespan_stops(timespan_1)
            False
            >>> timespan_1.stops_after_timespan_stops(timespan_2)
            False
            >>> timespan_2.stops_after_timespan_stops(timespan_1)
            True
            >>> timespan_2.stops_after_timespan_stops(timespan_2)
            False

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return expr_stop_offset < self_stop_offset

    def stops_at_offset(self, offset):
        r'''Is true when timespan stops at `offset`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.stops_at_offset(Offset(-5))
            False
            >>> timespan_1.stops_at_offset(Offset(0))
            False
            >>> timespan_1.stops_at_offset(Offset(5))
            False

        Returns true or false.
        '''
        offset = durationtools.Offset(offset)
        return self._stop_offset == offset

    def stops_at_or_after_offset(self, offset):
        r'''Is true when timespan stops at or after `offset`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.stops_at_or_after_offset(Offset(-5))
            True
            >>> timespan_1.stops_at_or_after_offset(Offset(0))
            True
            >>> timespan_1.stops_at_or_after_offset(Offset(5))
            True

        Returns true or false.
        '''
        offset = durationtools.Offset(offset)
        return offset <= self._stop_offset

    def stops_before_offset(self, offset):
        r'''Is true when timespan stops before `offset`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.stops_before_offset(Offset(-5))
            False
            >>> timespan_1.stops_before_offset(Offset(0))
            False
            >>> timespan_1.stops_before_offset(Offset(5))
            False

        Returns true or false.
        '''
        offset = durationtools.Offset(offset)
        return self._stop_offset < offset

    def stops_before_or_at_offset(self, offset):
        r'''Is true when timespan stops before or at `offset`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.stops_before_or_at_offset(Offset(-5))
            False
            >>> timespan_1.stops_before_or_at_offset(Offset(0))
            False
            >>> timespan_1.stops_before_or_at_offset(Offset(5))
            False

        Returns true or false.
        '''
        offset = durationtools.Offset(offset)
        return self._stop_offset <= offset

    def stops_before_timespan_starts(self, timespan):
        r'''Is true when timespan stops before `timespan` starts.
        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.stops_before_timespan_starts(timespan_1)
            False
            >>> timespan_1.stops_before_timespan_starts(timespan_2)
            False
            >>> timespan_2.stops_before_timespan_starts(timespan_1)
            False
            >>> timespan_2.stops_before_timespan_starts(timespan_2)
            False

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return self_stop_offset < expr_start_offset

    def stops_before_timespan_stops(self, timespan):
        r'''Is true when timespan stops before `timespan` stops. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.stops_before_timespan_stops(timespan_1)
            False
            >>> timespan_1.stops_before_timespan_stops(timespan_2)
            True
            >>> timespan_2.stops_before_timespan_stops(timespan_1)
            False
            >>> timespan_2.stops_before_timespan_stops(timespan_2)
            False

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return self_stop_offset < expr_stop_offset

    def stops_during_timespan(self, timespan):
        r'''Is true when timespan stops during `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.stops_during_timespan(timespan_1)
            True
            >>> timespan_1.stops_during_timespan(timespan_2)
            False
            >>> timespan_2.stops_during_timespan(timespan_1)
            False
            >>> timespan_2.stops_during_timespan(timespan_2)
            True

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            expr_start_offset < self_stop_offset and
            self_stop_offset <= expr_stop_offset
            )

    def stops_when_timespan_starts(self, timespan):
        r'''Is true when timespan stops when `timespan` starts. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.stops_when_timespan_starts(timespan_1)
            False
            >>> timespan_1.stops_when_timespan_starts(timespan_2)
            True
            >>> timespan_2.stops_when_timespan_starts(timespan_1)
            False
            >>> timespan_2.stops_when_timespan_starts(timespan_2)
            False

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return self_stop_offset == expr_start_offset

    def stops_when_timespan_stops(self, timespan):
        r'''Is true when timespan stops when `timespan` stops. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.stops_when_timespan_stops(timespan_1)
            True
            >>> timespan_1.stops_when_timespan_stops(timespan_2)
            False
            >>> timespan_2.stops_when_timespan_stops(timespan_1)
            False
            >>> timespan_2.stops_when_timespan_stops(timespan_2)
            True

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return self_stop_offset == expr_stop_offset

    def stretch(self, multiplier, anchor=None):
        r'''Stretches timespan by `multiplier` relative to `anchor`.

        .. container:: example

            **Example 1.** Stretch relative to timespan start offset:

            ::

                >>> timespantools.Timespan(3, 10).stretch(Multiplier(2))
                Timespan(start_offset=Offset(3, 1), stop_offset=Offset(17, 1))

        .. container:: example

            **Example 2.** Stretch relative to timespan stop offset:

            ::

                >>> timespantools.Timespan(3, 10).stretch(Multiplier(2), Offset(10))
                Timespan(start_offset=Offset(-4, 1), stop_offset=Offset(10, 1))

        .. container:: example

            **Example 3.** Stretch relative to offset prior to timespan:

            ::

                >>> timespantools.Timespan(3, 10).stretch(Multiplier(2), Offset(0))
                Timespan(start_offset=Offset(6, 1), stop_offset=Offset(20, 1))

        .. container:: example

            **Example 4.** Stretch relative to offset after timespan:

            ::

                >>> timespantools.Timespan(3, 10).stretch(Multiplier(3), Offset(12))
                Timespan(start_offset=Offset(-15, 1), stop_offset=Offset(6, 1))

        .. container:: example

            **Example 5.** Stretch relative to offset that happens during timespan:

            ::

                >>> timespantools.Timespan(3, 10).stretch(Multiplier(2), Offset(4))
                Timespan(start_offset=Offset(2, 1), stop_offset=Offset(16, 1))

        Returns newly emitted timespan.
        '''
        from abjad.tools.topleveltools import new
        multiplier = durationtools.Multiplier(multiplier)
        assert 0 < multiplier
        if anchor is None:
            anchor = self._start_offset
        new_start_offset = (multiplier * (self._start_offset - anchor)) + anchor
        new_stop_offset = (multiplier * (self._stop_offset - anchor)) + anchor
        result = new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return result

    def translate(self, translation=None):
        r'''Translates timespan by `translation`.

            >>> timespan = timespantools.Timespan(5, 10)

        ::

            >>> timespan.translate(2)
            Timespan(start_offset=Offset(7, 1), stop_offset=Offset(12, 1))

        Returns new timespan.
        '''
        return self.translate_offsets(translation, translation)

    def translate_offsets(
        self,
        start_offset_translation=None,
        stop_offset_translation=None,
        ):
        r'''Translates timespan start offset by `start_offset_translation` and
        stop offset by `stop_offset_translation`:

        ::

            >>> timespan = timespantools.Timespan((1, 2), (3, 2))

        ::

            >>> timespan.translate_offsets(
            ...     start_offset_translation=Duration(-1, 8))
            Timespan(start_offset=Offset(3, 8), stop_offset=Offset(3, 2))

        Returns new timespan.
        '''
        start_offset_translation = start_offset_translation or 0
        stop_offset_translation = stop_offset_translation or 0
        start_offset_translation = \
            durationtools.Duration(start_offset_translation)
        stop_offset_translation = \
            durationtools.Duration(stop_offset_translation)
        new_start_offset = self._start_offset + start_offset_translation
        new_stop_offset = self._stop_offset + stop_offset_translation
        result = new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return result

    def trisects_timespan(self, timespan):
        r'''Is true when timespan trisects `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 6)

        ::

            >>> timespan_1.trisects_timespan(timespan_1)
            False
            >>> timespan_1.trisects_timespan(timespan_2)
            False
            >>> timespan_2.trisects_timespan(timespan_1)
            True
            >>> timespan_2.trisects_timespan(timespan_2)
            False

        Returns true or false.
        '''
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            expr_start_offset < self_start_offset and
            self_stop_offset < expr_stop_offset
            )

    ### PUBLIC PROPERTIES ###

    @property
    def axis(self):
        r'''Arithmetic mean of timespan start- and stop-offsets.

        ::

            >>> timespantools.Timespan(0, 10).axis
            Offset(5, 1)

        Returns offset.
        '''
        return (self._start_offset + self._stop_offset) / 2

    @property
    def duration(self):
        r'''Duration of timespan.

        ::

            >>> timespantools.Timespan(0, 10).duration
            Duration(10, 1)

        Returns duration.
        '''
        return self._stop_offset - self._start_offset

    @property
    def is_closed(self):
        r'''Is false for all timespans.

        ::

            >>> timespantools.Timespan(0, 10).is_closed
            False

        Returns true or false.
        '''
        return BoundedObject.is_closed.fget(self)

    @property
    def is_half_closed(self):
        r'''Is true for all timespans.

        ::

            >>> timespantools.Timespan(0, 10).is_half_closed
            True

        Returns true or false.
        '''
        return BoundedObject.is_half_closed.fget(self)

    @property
    def is_half_open(self):
        r'''Is true for all timespans.

        ::

            >>> timespantools.Timespan(0, 10).is_half_open
            True

        Returns true or false.
        '''
        return BoundedObject.is_half_open.fget(self)

    @property
    def is_left_closed(self):
        r'''Is true for all timespans.

        ::

            >>> timespantools.Timespan(0, 10).is_left_closed
            True

        Returns true or false.
        '''
        return True

    @property
    def is_left_open(self):
        r'''Is false for all timespans.

        ::

            >>> timespantools.Timespan(0, 10).is_left_open
            False

        Returns true or false.
        '''
        return False

    @property
    def is_open(self):
        r'''Is false for all timespans.

        ::

            >>> timespantools.Timespan(0, 10).is_open
            False

        Returns true or false.
        '''
        return BoundedObject.is_open.fget(self)

    @property
    def is_right_closed(self):
        r'''Is false for all timespans.

        ::

            >>> timespantools.Timespan(0, 10).is_right_closed
            False

        Returns true or false.
        '''
        return False

    @property
    def is_right_open(self):
        r'''Is true for all timespans.

        ::

            >>> timespantools.Timespan(0, 10).is_right_open
            True

        Returns true or false.
        '''
        return True

    @property
    def is_well_formed(self):
        r'''Is true when timespan start offset preceeds timespan stop offset.
        Otherwise false:

        ::

            >>> timespantools.Timespan(0, 10).is_well_formed
            True

        Returns true or false.
        '''
        return self._start_offset < self._stop_offset

    @property
    def offsets(self):
        r'''Timespan offsets.

        ::

            >>> timespantools.Timespan(0, 10).offsets
            (Offset(0, 1), Offset(10, 1))

        Returns offset pair.
        '''
        return self._start_offset, self._stop_offset

    @property
    def start_offset(self):
        r'''Timespan start offset.

        ::

            >>> timespantools.Timespan(0, 10).start_offset
            Offset(0, 1)

        Returns offset.
        '''
        return self._start_offset

    @property
    def stop_offset(self):
        r'''Timespan stop offset.

        ::

            >>> timespantools.Timespan(0, 10).stop_offset
            Offset(10, 1)

        Returns offset.
        '''
        return self._stop_offset



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_BeatwiseQSchema___getitem___01():

    schema = quantizationtools.BeatwiseQSchema()

    assert schema[0] == schema[1] == schema[2] == {
        'beatspan': durationtools.Duration(1, 4),
        'search_tree': quantizationtools.UnweightedSearchTree(),
        'tempo': Tempo((1, 4), 60),
    }


def test_quantizationtools_BeatwiseQSchema___getitem___02():

    item_a = quantizationtools.BeatwiseQSchemaItem(
        search_tree=quantizationtools.UnweightedSearchTree({2: None}))
    item_b = quantizationtools.BeatwiseQSchemaItem(tempo=((1, 4), 76))
    item_c = quantizationtools.BeatwiseQSchemaItem(beatspan=(1, 8),
        search_tree=quantizationtools.UnweightedSearchTree({5: None}))

    schema = quantizationtools.BeatwiseQSchema(
        {2: item_a, 4: item_b, 7: item_c},
        beatspan=durationtools.Duration(1, 32),
        search_tree=quantizationtools.UnweightedSearchTree({3: None}),
        tempo=Tempo((1, 16), 36)
        )

    assert schema[0] == schema[1] == {
        'beatspan': durationtools.Duration(1, 32),
        'search_tree': quantizationtools.UnweightedSearchTree({3: None}),
        'tempo': Tempo((1, 16), 36),
    }

    assert schema[2] == schema[3] == {
        'beatspan': durationtools.Duration(1, 32),
        'search_tree': quantizationtools.UnweightedSearchTree({2: None}),
        'tempo': Tempo((1, 16), 36),
    }

    assert schema[4] == schema[5] == schema[6] == {
        'beatspan': durationtools.Duration(1, 32),
        'search_tree': quantizationtools.UnweightedSearchTree({2: None}),
        'tempo': Tempo((1, 4), 76),
    }

    assert schema[7] == schema[8] == schema[1000] == {
        'beatspan': durationtools.Duration(1, 8),
        'search_tree': quantizationtools.UnweightedSearchTree({5: None}),
        'tempo': Tempo((1, 4), 76),
    }



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_MultipartBeam___init___01():
    r'''Initialize empty multipart beam spanner.
    '''

    beam = spannertools.MultipartBeam()
    assert isinstance(beam, spannertools.MultipartBeam)



# -*- coding: utf-8 -*-
import copy
from abjad.tools import durationtools
from abjad.tools.scoretools.Leaf import Leaf
from abjad.tools.topleveltools import detach


class Rest(Leaf):
    r'''A rest.

    ..  container:: example

        ::

            >>> rest = Rest('r8.')
            >>> measure = Measure((3, 16), [rest])
            >>> show(measure) # doctest: +SKIP

        ..  doctest::

            >>> print(format(measure))
            {
                \time 3/16
                r8.
            }

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Leaves'

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, written_duration=None):
        from abjad.tools.topleveltools import parse
        original_input = written_duration
        if isinstance(written_duration, str):
            string = '{{ {} }}'.format(written_duration)
            parsed = parse(string)
            assert len(parsed) == 1 and isinstance(parsed[0], Leaf)
            written_duration = parsed[0]
        if isinstance(written_duration, Leaf):
            written_duration = written_duration.written_duration
        elif written_duration is None:
            written_duration = durationtools.Duration(1, 4)
        else:
            written_duration = durationtools.Duration(written_duration)
        Leaf.__init__(self, written_duration)
        if isinstance(original_input, Leaf):
            self._copy_override_and_set_from_leaf(original_input)

    ### PRIVATE METHODS ###

    def _divide(self, pitch=None):
        from abjad.tools import markuptools
        treble = copy.copy(self)
        bass = copy.copy(self)
        detach(markuptools.Markup, treble)
        detach(markuptools.Markup, bass)
        up_markup = self._get_markup(direction=Up)
        up_markup = [copy.copy(markup) for markup in up_markup]
        down_markup = self._get_markup(direction=Down)
        down_markup = [copy.copy(markup) for markup in down_markup]
        for markup in up_markup:
            markup(treble)
        for markup in down_markup:
            markup(bass)
        return treble, bass

    ### PRIVATE PROPERTIES ###

    @property
    def _body(self):
        return [self._compact_representation]

    @property
    def _compact_representation(self):
        return 'r{}'.format(self._formatted_duration)



# -*- coding: utf-8 -*-
from abjad import *


def test_lilypondproxytools_LilyPondGrobNameManager___delattr___01():

    note = Note("c'4")
    override(note).accidental.color = 'red'
    override(note).beam.positions = (-6, -6)
    override(note).dots.thicknes = 2

    assert format(note) == stringtools.normalize(
        r'''
        \once \override Accidental.color = #red
        \once \override Beam.positions = #'(-6 . -6)
        \once \override Dots.thicknes = #2
        c'4
        '''
        )

    del(override(note).accidental)
    del(override(note).beam)

    assert format(note) == stringtools.normalize(
        r'''
        \once \override Dots.thicknes = #2
        c'4
        '''
        )


def test_lilypondproxytools_LilyPondGrobNameManager___delattr___02():
    r'''Delete LilyPond Rest grob override.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    override(staff).rest.transparent = True
    del(override(staff).rest)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )


def test_lilypondproxytools_LilyPondGrobNameManager___delattr___03():
    r'''Delete LilyPond TimeSignature grob override.
    '''

    note = Note("c'4")
    override(note).time_signature.color = 'red'
    override(note).time_signature.transparent = True
    del(override(note).time_signature)

    assert format(note) == "c'4"



# -*- coding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_ScaleDegree__initialize_by_number_01():

    degree = tonalanalysistools.ScaleDegree(2)
    assert degree.accidental == pitchtools.Accidental('')
    assert degree.number == 2


def test_tonalanalysistools_ScaleDegree__initialize_by_number_02():
    r'''Initialize from other scale degreeinstance.
    '''

    degree = tonalanalysistools.ScaleDegree(2)
    new = tonalanalysistools.ScaleDegree(degree)

    assert degree is not new
    assert new.number == 2



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_indicatortools_Tempo_attach_01():

    score = Score(r"\new Staff { c'' d'' e'' f'' } \new Staff { c' d' e' f' }")
    tempo_1 = Tempo((1, 8), 52)
    tempo_2 = Tempo((1, 8), 73)
    attach(tempo_1, score[0][0])

    assert pytest.raises(Exception, 'attach(tempo_2, score[1][0])')



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_IterationAgent_by_timeline_from_component_01():
    r'''Yields component class instances in score of argument,
    sorted backward by score offset stop time,
    and starting from argument.
    '''

    score = Score()
    staff_1 = Staff("c'4 d'4 e'4 f'4")
    staff_2 = Staff("g'8 a'8 b'8 c''8")
    score.extend([staff_1, staff_2])

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                c'4
                d'4
                e'4
                f'4
            }
            \new Staff {
                g'8
                a'8
                b'8
                c''8
            }
        >>
        '''
        )

    leaf_generator = iterate(staff_2[2]).by_timeline_from_component(
        reverse=True,
        )
    leaves = list(leaf_generator)

    assert leaves[0] is staff_2[2] # b'8
    assert leaves[1] is staff_1[0] # c'4
    assert leaves[2] is staff_2[1] # a'8
    assert leaves[3] is staff_2[0] # g'8


def test_agenttools_IterationAgent_by_timeline_from_component_02():
    r'''Yields component class instances in score of argument,
    sorted backward by score offset stop time,
    and starting from argument.
    '''

    score = Score()
    staff_1 = Staff("c'8 d'8 e'8 f'8")
    staff_2 = Staff("g'4 a'4 b'4 c''4")
    score.extend([staff_1, staff_2])

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
            \new Staff {
                g'4
                a'4
                b'4
                c''4
            }
        >>
        '''
        )

    leaf_generator = iterate(staff_1[3]).by_timeline_from_component(
        reverse=True,
        )
    leaves = list(leaf_generator)

    assert leaves[0] is staff_1[3] # f'8
    assert leaves[1] is staff_2[1] # a'4
    assert leaves[2] is staff_1[2] # e'8
    assert leaves[3] is staff_1[1] # d'8
    assert leaves[4] is staff_2[0] # g'4
    assert leaves[5] is staff_1[0] # c'8


def test_agenttools_IterationAgent_by_timeline_from_component_03():
    r'''Yields component class instances in score of argument,
    sorted by score offset and score index,
    and starting from argument.
    '''

    score = Score()
    staff_1 = Staff("c'4 d'4 e'4 f'4")
    staff_2 = Staff("g'8 a'8 b'8 c''8")
    score.extend([staff_1, staff_2])

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                c'4
                d'4
                e'4
                f'4
            }
            \new Staff {
                g'8
                a'8
                b'8
                c''8
            }
        >>
        '''
        )

    leaf_generator = iterate(staff_2[2]).by_timeline_from_component()
    leaves = list(leaf_generator)

    assert leaves[0] is staff_2[2] # b'8
    assert leaves[1] is staff_2[3] # c''8
    assert leaves[2] is staff_1[2] # e'4
    assert leaves[3] is staff_1[3] # f'4


def test_agenttools_IterationAgent_by_timeline_from_component_04():
    r'''Yields component class instances in score of argument,
    sorted by score offset and score index,
    and starting from argument.
    '''

    score = Score()
    staff_1 = Staff("c'8 d'8 e'8 f'8")
    staff_2 = Staff("g'4 a'4 b'4 c''4")
    score.extend([staff_1, staff_2])

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
            \new Staff {
                g'4
                a'4
                b'4
                c''4
            }
        >>
        '''
        )

    leaf_generator = iterate(staff_1[3]).by_timeline_from_component()
    leaves = list(leaf_generator)

    assert leaves[0] is staff_1[3] # f'8
    assert leaves[1] is staff_2[2] # b'4
    assert leaves[2] is staff_2[3] # c''4



# -*- coding: utf-8 -*-


def is_positive_integer_equivalent_number(argument):
    '''Is true when `argument` is a positive integer-equivalent number.
    Otherwise false.

    ..  container:: example

        ::

            >>> mathtools.is_positive_integer_equivalent_number(Duration(4, 2))
            True

    Returns true or false.
    '''
    from abjad.tools import mathtools
    try:
        return (
            0 < argument and
            mathtools.is_integer_equivalent_number(argument)
            )
    except TypeError:  # Python 3 comparisons with non-numbers
        return False



# -*- coding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_Chord_written_pitches_01():
    r'''Returns immutable tuple of pitches in chord.
    '''

    chord = Chord("<d' e' f'>4")
    pitches = chord.written_pitches

    assert isinstance(pitches, pitchtools.PitchSegment)
    assert len(pitches) == 3
    assert pytest.raises(Exception, 'pitches.pop()')
    assert pytest.raises(Exception, 'pitches.remove(pitches[0])')


def test_scoretools_Chord_written_pitches_02():
    r'''Equivalent written pitches compare equal.
    '''

    chord_1 = Chord("<d' e' f'>4")
    chord_2 = Chord("<d' e' f'>4")

    assert chord_1.written_pitches == chord_2.written_pitches


def test_scoretools_Chord_written_pitches_03():
    '''Transpose sounding pitches to written pitches.
    '''

    staff = Staff("<c''' e'''>4 <d''' fs'''>4")
    glockenspiel = instrumenttools.Glockenspiel()
    attach(glockenspiel, staff)
    instrumenttools.transpose_from_sounding_pitch_to_written_pitch(staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \set Staff.instrumentName = \markup { Glockenspiel }
            \set Staff.shortInstrumentName = \markup { Gkspl. }
            <c' e'>4
            <d' fs'>4
        }
        ''')

    assert staff[0].written_pitches == (
        NamedPitch("c'"),
        NamedPitch("e'"),
        )


def test_scoretools_Chord_written_pitches_04():
    r'''Set written pitches with pitch numbers.
    '''

    chord = Chord([], (1, 4))
    chord.written_pitches = [4, 3, 2]
    assert format(chord) == "<d' ef' e'>4"

    chord.written_pitches = (4, 3, 2)
    assert format(chord) == "<d' ef' e'>4"


def test_scoretools_Chord_written_pitches_05():
    r'''Set written pitches with pitches.
    '''

    chord = Chord([], (1, 4))
    chord.written_pitches = [
        NamedPitch(4),
        NamedPitch(3),
        NamedPitch(2),
        ]

    assert format(chord) == "<d' ef' e'>4"


def test_scoretools_Chord_written_pitches_06():
    r'''Set written pitches with both pitches and pitch numbers.
    '''

    chord = Chord([], (1, 4))
    chord.written_pitches = [
        4,
        NamedPitch(3),
        NamedPitch(2),
        ]

    assert format(chord) == "<d' ef' e'>4"



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_NumberedPitchClass___sub___01():
    r'''Subtracting one pitch-class from another.
    '''

    pitch_class_1 = pitchtools.NumberedPitchClass(6)
    pitch_class_2 = pitchtools.NumberedPitchClass(7)

    assert pitch_class_1 - pitch_class_2 == \
        pitchtools.NumberedInversionEquivalentIntervalClass(1)
    assert pitch_class_2 - pitch_class_1 == \
        pitchtools.NumberedInversionEquivalentIntervalClass(1)


def test_pitchtools_NumberedPitchClass___sub___02():
    r'''Subtracting an interval-class from a pitch-class.
    '''

    pitch_class = pitchtools.NumberedPitchClass(0)
    interval_class = pitchtools.NumberedInversionEquivalentIntervalClass(2)

    assert pitch_class - interval_class == pitchtools.NumberedPitchClass(10)
    assert pytest.raises(TypeError, 'interval_class - pitch_class')



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedIntervalClass___init___01():
    r'''Unisons and octaves are treated differently.
    '''

    mdic = pitchtools.NamedIntervalClass('perfect', -15)
    assert str(mdic) == '-P8'

    mdic = pitchtools.NamedIntervalClass('perfect', -8)
    assert str(mdic) == '-P8'

    mdic = pitchtools.NamedIntervalClass('perfect', 8)
    assert str(mdic) == '+P8'

    mdic = pitchtools.NamedIntervalClass('perfect', 15)
    assert str(mdic) == '+P8'


def test_pitchtools_NamedIntervalClass___init___02():
    r'''Unisons and octaves are treated differently.
    '''

    mdic = pitchtools.NamedIntervalClass('perfect', -1)
    assert str(mdic) == 'P1'

    mdic = pitchtools.NamedIntervalClass('perfect', 1)
    assert str(mdic) == 'P1'



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_NumberedPitch___int___01():
    r'''Returns pitch number of 12-ET numbered pitch as int.
    '''

    numbered_pitch = pitchtools.NumberedPitch(13)
    assert isinstance(int(numbered_pitch), int)
    assert int(numbered_pitch) == 13


def test_pitchtools_NumberedPitch___int___02():
    r'''Raise type error on non-12-ET numbered pitch.
    '''

    numbered_pitch = pitchtools.NumberedPitch(13.5)
    assert pytest.raises(TypeError, 'int(numbered_pitch)')
